use std::ops::DerefMut;

use crate::{
    component::Component,
    input::InputState,
    layout::{LayoutCache, LayoutNode, LayoutNodeAccess, LeafLayoutNode, SizePolicy},
    ui::UIHandle,
};

/*mod button;
mod frame;
mod group;
mod label;

pub use frame::Frame;
pub use group::PlainGroup;
pub use label::Label;*/

pub trait Widget<C: Component> {
    fn poll(&mut self, input_state: Option<&InputState>) -> Vec<C::Msg>;

    fn layout_node(&self) -> LayoutNodeAccess;
    fn layout_node_mut(&mut self) -> &mut LayoutNode;
    fn render(&self, target: &mut tiny_skia::Pixmap, ti: &());
}

pub struct Spacer<C: Component> {
    lnode: LeafLayoutNode,
    _ghost: std::marker::PhantomData<C>,
}

impl<C: Component> Spacer<C> {
    pub fn new(uih: &UIHandle) -> Self {
        let mut lnode = LeafLayoutNode::new(uih.new_layout_node());
        lnode.set_width_policy(SizePolicy::expands(1)).set_height_policy(SizePolicy::expands(1));
        Self {
            lnode,
            _ghost: Default::default(),
        }
    }
}

impl<C: Component> Widget<C> for Spacer<C> {
    fn poll(&mut self, input_state: Option<&InputState>) -> Vec<<C as Component>::Msg> {
        vec![]
    }

    fn layout_node(&self) -> LayoutNodeAccess {
        LayoutNodeAccess::new(&self.lnode)
    }
    fn layout_node_mut(&mut self) -> &mut LayoutNode {
        self.lnode.deref_mut()
    }

    fn render(&self, target: &mut tiny_skia::Pixmap, ti: &()) {
        let mut path = tiny_skia::PathBuilder::new();
        path.push_rect(self.layout_node().cached_rect().unwrap().to_rect());
        let path = path.finish().unwrap();
        let mut paint = tiny_skia::Paint::default();
        paint.set_color_rgba8(192, 168, 32, 255);
        target.fill_path(
            &path,
            &paint,
            tiny_skia::FillRule::Winding,
            tiny_skia::Transform::identity(),
            None,
        );
        target.fill(tiny_skia::Color::from_rgba(1.0, 0.5, 0.5, 1.0).unwrap());
    }
}
